package kawigi.tools;
import java.io.*;

/**
 * FileFormat is the interface that should be implemented to read in a bots data.  It should have
 * a unique name (to be chosen from a list) and a method to implement reading the data and
 * putting it into a SegmentedData object.  For a simple implementation example, see
 * FloodMini14Format.java.  Note that a GrapherSegmentation implementation is required for
 * every segmentation axis, but most segmentations can probably use a variant of one already
 * written.
 **/
public interface FileFormat
{
	/**
	 * Reads the data from the file and creates from it a SegmentedData object, consisting
	 * of a Scheme and an array of Segmentations.  Throw an InvalidFileException if the format
	 * of the given file is wrong.
	 **/
	public SegmentedData getData(File datafile) throws IOException, InvalidFileException;
	
	/**
	 * Returns the logical name of this format.  Should be unique from other names.
	 **/
	public String getFormatName();
}
