package kawigi.tools;
import robocode.*;
import kawigi.tools.fs.*;
import kawigi.tools.parts.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;

public class FloodGrapher extends AdvancedRobot
{
	private static Object stats;
	private static Segmentation[] segmentations;
	private static Movement movement;
	private static Vector waves;
	private static PowerManagement pm;
	private static int direction = 1, guessfactors = 31, savePeriod = 5;
	private static boolean constantwaves, loadData;
	private static String name;
	
	public void run()
	{
		setAdjustGunForRobotTurn(true);
		setAdjustRadarForGunTurn(true);
		waves = new Vector();
		if (getRoundNum() == 0)
		{
			FloodScriptParser parser = new FloodScriptParser(this);
			parser.loadSettings(getDataFile("robot.ini"));
			movement = (Movement)parser.getObject("movement");
			pm = (PowerManagement)parser.getObject("powermanagement");
			constantwaves = ((Boolean)parser.getObject("continuous_waves", new Boolean(false))).booleanValue();
			guessfactors = ((Number)parser.getObject("guess_factors", new Integer(31))).intValue();
			Vector segs = parser.getObjectList("segmentation");
			segmentations = new Segmentation[segs.size()];
			segs.toArray(segmentations);
			savePeriod = ((Number)parser.getObject("save_period", new Integer(10))).intValue();
			loadData = ((Boolean)parser.getObject("load_data", new Boolean(true))).booleanValue();
			stats = createStats(0);
		}
		setTurnRadarRight(Double.POSITIVE_INFINITY);
		do
		{
			movement.run(this);
			execute();
		}
		while (true);
	}
	
	public void onEvent(Event e)
	{
		movement.onEvent(e, this);
	}
	
	public void onScannedRobot(ScannedRobotEvent e)
	{
		if (name == null)
		{
			name = e.getName();
			if (loadData)
				try
				{
					ObjectInputStream in = new ObjectInputStream(new GZIPInputStream(new FileInputStream(getDataFile(name))));
					stats = in.readObject();
					segmentations = (Segmentation[])in.readObject();
					in.close();
				}
				catch (Exception ex)
				{
				}
		}
		onEvent(e);
		setTurnRadarLeft(getRadarTurnRemaining());
		double[] current = getCurrentSegment(e, stats, 0);
		double latv = Math.sin(e.getHeadingRadians()-getHeadingRadians()-e.getBearingRadians())*e.getVelocity();
		if (latv != 0)
			direction = (latv < 0) ? -1 : 1;
		
		double ex = getX()+Math.sin(getHeadingRadians()+e.getBearingRadians())*e.getDistance();
		double ey = getY()+Math.cos(getHeadingRadians()+e.getBearingRadians())*e.getDistance();
		for (int i=0; i<waves.size(); i++)
			if (((WaveBullet)waves.elementAt(i)).update(ex, ey, getTime()))
				waves.removeElementAt(i);
		
		double power = pm.getPower(e, this);
		int bestindex = current.length/2;
		if (e.getEnergy() > 0)
			for (int i=0; i<current.length; i++)
				if (current[i] > current[bestindex])
					bestindex = i--;
		
		setTurnGunRightRadians(robocode.util.Utils.normalRelativeAngle(getHeadingRadians()+e.getBearingRadians()-getGunHeadingRadians()+(Math.asin(8/(20-power*3))*direction)*(bestindex*2.0/current.length-1)));
		
		if (Math.abs(getGunTurnRemaining()) < 5 && (setFireBullet(power) != null || constantwaves))
			waves.add(new WaveBullet(getX(), getY(), direction, 20-power*3, getX()+Math.sin(getHeadingRadians()+e.getBearingRadians())*e.getDistance(), getY()+Math.cos(getHeadingRadians()+e.getBearingRadians())*e.getDistance(), getTime(), current));
		
	}
	
	public void onHitByBullet(HitByBulletEvent e)
	{
		onEvent(e);
	}
	
	public void onBulletHit(BulletHitEvent e)
	{
		onEvent(e);
	}
	
	public void onBulletMissed(BulletMissedEvent e)
	{
		onEvent(e);
	}
	
	public void onBulletHitBullet(BulletHitBulletEvent e)
	{
		onEvent(e);
	}
	
	public void onDeath(DeathEvent e)
	{
		onEvent(e);
		onEnd();
	}
	
	public void onHitWall(HitWallEvent e)
	{
		onEvent(e);
	}
	
	public void onRobotDeath(RobotDeathEvent e)
	{
		onEvent(e);
	}
	
	public void onWin(WinEvent e)
	{
		onEvent(e);
		onEnd();
	}
	
	public void onSkippedTurn(SkippedTurnEvent e)
	{
		onEvent(e);
	}
	
	public void onCustomEvent(CustomEvent e)
	{
		onEvent(e);
	}
	
	public void onEnd()
	{
		if (getRoundNum() == getNumRounds()-1 || getRoundNum()%savePeriod == 0)
		{
			try
			{
				ObjectOutputStream out = new ObjectOutputStream(new GZIPOutputStream(new RobocodeFileOutputStream(getDataFile(name))));
				out.writeObject(stats);
				out.writeObject(segmentations);
				out.close();
			}
			catch (IOException ex)
			{
			}
		}
	}

	public double[] getCurrentSegment(ScannedRobotEvent e, Object stats, int index)
	{
		if (index < segmentations.length)
			return getCurrentSegment(e, ((Object[])stats)[segmentations[index].getSegmentIndex(e, this, pm)], index+1);
		else
			return (double[])stats;
	}
	
	public Object createStats(int index)
	{
		if (index == segmentations.length)
			return new double[guessfactors];
		else
		{
			int numSegmentations = segmentations[index].numSegments();
			Object[] stuff = new Object[numSegmentations];
			for (int i=0; i<numSegmentations; i++)
				stuff[i] = createStats(index+1);
			return stuff;
		}
	}
}
