package kawigi.tools;
import javax.swing.*;

/**
 * If you want to create your own Tab in the FloodGrapher application, it should extend
 * this base class and configured either by editing floodgrapher.ini or using the Plugin manager
 * FloodGrapherPane, PluginManagerPanel, FloodScriptEditorPane and HelpPanel are some subclasses
 * of FloodGrapherGUIPlugin that may be used as examples.
 **/
public abstract class FloodGrapherGUIPlugin extends JPanel
{
	/**
	 * Returns the name of the plugin, which will be used by the Plugin manager
	 * and as the title to the tab for this plugin.
	 **/
	public abstract String getName();
	
	/**
	 * Returns the section of FloodScript code that should be used to initialize this
	 * object roughly in the same state as it is currently in.  This should contain
	 * a line that starts with "plugin=" somewhere.
	 **/
	public abstract String getFloodScriptCommand();
	
	/**
	 * Subclasses of FloodGrapherGUIPlugin are created while the FloodScript is being run.
	 * This is called after the script runs and the Application has basically been started.
	 * Override this if you want to do any other setup that depends on the FloodGrapher GUI
	 * being completely loaded.
	 **/
	public void initialize()
	{
	}
}
