package kawigi.tools;
import kawigi.tools.graph.*;

/**
 * This interface should be implemented for each segmentation axis of a robot.  The main purpose is
 * to control segments selected through the GUI.  For implementation examples, see
 * AccelerationGSegmentation.java, BFTGSegmentation.java, LateralVelocityGSegmentation.java
 * and WallGSegmentation.java
 **/
public interface GrapherSegmentation
{
	/**
	 * Returns the Logical name of this segmentation, so the user knows what they're selecting.
	 **/
	public String getName();
	/**
	 * Adds any components needed to select segments for this segmentation.  The window may
	 * also be assigned as an ActionListener or ItemListener for this item.  In many cases,
	 * this will simply create a JLabel and a JComboBox and add it to a JPanel and add the
	 * JPanel to the window.  You should also keep a reference to widgets added here and never
	 * create more than one for the same object (add the old one if it has already been created)
	 **/
	public void addSelectionWidget(GraphControlWindow window);
	/**
	 * Returns true if the segment at that index should be used according the current state of
	 * the widgets added by addSelectionWidget.  This should never be called before
	 * addSelectionWidget.
	 **/
	public boolean accept(int index);
}
