package kawigi.tools;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

public class PluginManagerPanel extends FloodGrapherGUIPlugin implements ActionListener
{
	private Vector currentFormats, formatStrings, plugins, pluginStrings;
	private JList formatList, pluginList;
	private JButton addFormatButton, removeFormatButton, addPluginButton, removePluginButton;
	private JMenuItem newFormatItem, newPluginItem;
	private FloodGrapherGUI parent;
	
	public PluginManagerPanel(FloodGrapherGUI window)
	{
		parent = window;
		newFormatItem = new JMenuItem("Add Format Plugin...");
		newFormatItem.addActionListener(this);
		newPluginItem = new JMenuItem("Add GUI Plugin...");
		newPluginItem.addActionListener(this);
		parent.getJMenuBar().getMenu(0).add(newFormatItem);
		parent.getJMenuBar().getMenu(0).add(newPluginItem);
		parent.getJMenuBar().getMenu(0).addSeparator();
		
		formatList = new JList();
		addFormatButton = new JButton("Add");
		removeFormatButton = new JButton("Remove");
		addFormatButton.addActionListener(this);
		removeFormatButton.addActionListener(this);
		JPanel listPanel = new JPanel(new GridLayout(1, 1));
		listPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "Current File Formats"));
		listPanel.add(formatList);
		JPanel formatPanel = new JPanel(new BorderLayout());
		formatPanel.add(listPanel);
		JPanel buttonPanel = new JPanel();
		buttonPanel.add(addFormatButton);
		buttonPanel.add(removeFormatButton);
		buttonPanel.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		formatPanel.add(buttonPanel, BorderLayout.SOUTH);
		
		pluginList = new JList();
		addPluginButton = new JButton("Add");
		removePluginButton = new JButton("Remove");
		addPluginButton.addActionListener(this);
		removePluginButton.addActionListener(this);
		JPanel pluginListPanel = new JPanel(new GridLayout(1, 1));
		pluginListPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), "Current Plugins"));
		pluginListPanel.add(pluginList);
		JPanel pluginPanel = new JPanel(new BorderLayout());
		pluginPanel.add(pluginListPanel);
		JPanel buttonPanel2 = new JPanel();
		buttonPanel2.add(addPluginButton);
		buttonPanel2.add(removePluginButton);
		buttonPanel2.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		pluginPanel.add(buttonPanel2, BorderLayout.SOUTH);
		
		setLayout(new GridLayout(1, 2));
		add(formatPanel);
		add(pluginPanel);
	}
	
	public void initialize()
	{
		currentFormats = parent.getFormatList();
		formatStrings = new Vector();
		for (int i=0; i<currentFormats.size(); i++)
			formatStrings.add(((FileFormat)currentFormats.elementAt(i)).getFormatName());
		formatList.setListData(formatStrings);
		
		plugins = parent.getPlugins();
		pluginStrings = new Vector();
		for (int i=0; i<plugins.size(); i++)
			pluginStrings.add(((FloodGrapherGUIPlugin)plugins.elementAt(i)).getName());
		pluginList.setListData(pluginStrings);
	}
	
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == removeFormatButton)
		{
			if (formatList.getSelectedIndex() >= 0)
			{
				currentFormats.removeElementAt(formatList.getSelectedIndex());
				formatStrings.removeElementAt(formatList.getSelectedIndex());
				formatList.setListData(formatStrings);
			}
		}
		else if (e.getSource() == addFormatButton || e.getSource() == newFormatItem)
		{
			parent.getTabbedPane().setSelectedComponent(this);
			String classname = JOptionPane.showInputDialog(this, "What is the FloodScript command to create this plugin?");
			if (classname != null)
				try
				{
					FileFormat ff = (FileFormat)parent.getFloodScriptParser().createObjectFromCommand(classname);
					currentFormats.addElement(ff);
					formatStrings.addElement(ff.getFormatName());
					formatList.setListData(formatStrings);
				}
				catch (ExceptionInInitializerError ex)
				{
					JOptionPane.showMessageDialog(this, "Error initializing object: " + ex.getException().getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
				}
				catch (LinkageError ex)
				{
					JOptionPane.showMessageDialog(this, "Linkage Error while loading class.", "Error", JOptionPane.ERROR_MESSAGE);
				}
				catch (ClassNotFoundException ex)
				{
					JOptionPane.showMessageDialog(this, "Class not found.  Is it in the current classpath and did you give the full classname (w/ package)?", "Error", JOptionPane.ERROR_MESSAGE);
				}
				catch (ClassCastException ex)
				{
					JOptionPane.showMessageDialog(this, "This class does not implement the FileFormat interface.", "Error", JOptionPane.ERROR_MESSAGE);
				}
				catch (InstantiationException ex)
				{
					JOptionPane.showMessageDialog(this, "Class is not instantiable or has no parameterless constructor.", "Error", JOptionPane.ERROR_MESSAGE);
				}
				catch (SecurityException ex)
				{
					JOptionPane.showMessageDialog(this, "Insufficient permission to dynamically load this class.", "Error", JOptionPane.ERROR_MESSAGE);
				}
				catch (IllegalAccessException ex)
				{
					JOptionPane.showMessageDialog(this, "Class or parameterless constructor is inaccessible.", "Error", JOptionPane.ERROR_MESSAGE);
				}
				catch (Exception ex)
				{
					JOptionPane.showMessageDialog(this, "An Error Occured:" + ex, "Error", JOptionPane.ERROR_MESSAGE);
				}
		}
		else if (e.getSource() == removePluginButton)
		{
			if (pluginList.getSelectedIndex() >= 0)
			{
				plugins.removeElementAt(pluginList.getSelectedIndex());
				pluginStrings.removeElementAt(pluginList.getSelectedIndex());
				pluginList.setListData(pluginStrings);
			}
		}
		else if (e.getSource() == addPluginButton || e.getSource() == newPluginItem)
		{
			parent.getTabbedPane().setSelectedComponent(this);
			String classname = JOptionPane.showInputDialog(this, "What is the FloodScript command to create this plugin?");
			if (classname != null)
				try
				{
					FloodGrapherGUIPlugin newPlugin = (FloodGrapherGUIPlugin)parent.getFloodScriptParser().createObjectFromCommand(classname);
					plugins.addElement(newPlugin);
					pluginStrings.addElement(newPlugin.getName());
					pluginList.setListData(pluginStrings);
					newPlugin.initialize();
					parent.getTabbedPane().add(newPlugin.getName(), newPlugin);
				}
				catch (ExceptionInInitializerError ex)
				{
					JOptionPane.showMessageDialog(this, "Error initializing object: " + ex.getException().getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
				}
				catch (LinkageError ex)
				{
					JOptionPane.showMessageDialog(this, "Linkage Error while loading class.", "Error", JOptionPane.ERROR_MESSAGE);
				}
				catch (ClassNotFoundException ex)
				{
					JOptionPane.showMessageDialog(this, "Class not found.  Is it in the current classpath and did you give the full classname (w/ package)?", "Error", JOptionPane.ERROR_MESSAGE);
				}
				catch (ClassCastException ex)
				{
					JOptionPane.showMessageDialog(this, "This class does not extend FloodGrapherGUIPlugin.", "Error", JOptionPane.ERROR_MESSAGE);
				}
				catch (InstantiationException ex)
				{
					JOptionPane.showMessageDialog(this, "Class is not instantiable or has no parameterless constructor.", "Error", JOptionPane.ERROR_MESSAGE);
				}
				catch (SecurityException ex)
				{
					JOptionPane.showMessageDialog(this, "Insufficient permission to dynamically load this class.", "Error", JOptionPane.ERROR_MESSAGE);
				}
				catch (IllegalAccessException ex)
				{
					JOptionPane.showMessageDialog(this, "Class or parameterless constructor is inaccessible.", "Error", JOptionPane.ERROR_MESSAGE);
				}
				catch (Exception ex)
				{
					JOptionPane.showMessageDialog(this, "An Error Occured:" + ex, "Error", JOptionPane.ERROR_MESSAGE);
				}
		}
	}
	
	public String getName()
	{
		return "Plugins";
	}
	
	public String getFloodScriptCommand()
	{
		return	  "#The plugin configurator:\n"
			+ "plugin=kawigi.tools.PluginManagerPanel(this)\n";
	}
}
