package kawigi.tools;
import kawigi.tools.graph.*;

/**
 * Represents the combination of a Scheme and a multi-dimensional array of doubles, whose
 * dimensionality is one more than the size of the Scheme.  This is the normalized, universal 
 * "stat buffer".  If the default format of the saved data is not a multi-dimensionalarray of
 *  doubles, it should be converted into one to create this object.
 **/
public class SegmentedData
{
	private Scheme scheme;
	private Object statsbuffer;
	
	/**
	 * Creates a new SegmentedData object
	 **/
	public SegmentedData(Scheme scheme, Object stats)
	{
		this.scheme = scheme;
		statsbuffer = stats;
	}
	
	/**
	 * Creates the GUI according the GrapherSegmentations in the Scheme to manipulate the data.
	 **/
	public void createSelectionGUI(GraphControlWindow window)
	{
		for (int i=0; i<scheme.size(); i++)
			scheme.segmentAt(i).addSelectionWidget(window);
	}
	
	/**
	 * Gets the specific array of values for the currently selected portion of the graph.
	 * No color included, this is added by the control window.
	 **/
	public GraphData getSelectedGraphData()
	{
		return new GraphData(recGetSelectedData(statsbuffer, 0));
	}
	
	private double[] recGetSelectedData(Object stats, int segmentationIndex)
	{
		if (segmentationIndex >= scheme.size())
			return (double[])stats;
		else
		{
			GrapherSegmentation seg = scheme.segmentAt(segmentationIndex);
			Object[] statsarray = (Object[])stats;
			double[] ret = null;
			for (int i=0; i<statsarray.length; i++)
			{
				if (seg.accept(i))
				{
					if (ret == null)
						ret = recGetSelectedData(statsarray[i], segmentationIndex+1);
					else
						ret = add(ret, recGetSelectedData(statsarray[i], segmentationIndex+1));
				}
			}
			return ret;
		}
	}
	
	private double[] add(double[] array1, double[] array2)
	{
		double[] ret = new double[Math.min(array1.length, array2.length)];
		for (int i=0; i<ret.length; i++)
			ret[i] = array1[i] + array2[i];
		return ret;
	}
	
	/**
	 * Set this object's statbuffer to that of reloaded.  Used to reload data from a file.
	 **/
	public void setData(SegmentedData reloaded)
	{
		statsbuffer = reloaded.statsbuffer;
	}
}
