package kawigi.tools.codesize;
import kawigi.tools.*;
import java.io.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import codesize.*;

public class CodesizePane extends FloodGrapherGUIPlugin implements ActionListener, TableModel
{
	private JMenuItem newWatchItem;
	private JButton addWatchButton, removeWatchButton, clearButton, refreshButton;
	private FloodGrapherGUI parent;
	private JFileChooser fileChooser;
	private JTable table;
	private Vector tmlisteners, watches;
	
	public CodesizePane(FloodGrapherGUI parent)
	{
		this.parent = parent;
		tmlisteners = new Vector();
		watches = new Vector();
		setLayout(new BorderLayout());
		newWatchItem = new JMenuItem("New Codesize watch...");
		parent.getJMenuBar().getMenu(0).add(newWatchItem);
		parent.getJMenuBar().getMenu(0).addSeparator();
		newWatchItem.addActionListener(this);
		
		fileChooser = new JFileChooser(".");
		fileChooser.addChoosableFileFilter(new ClassFileFilter());
		fileChooser.setAcceptAllFileFilterUsed(false);
		fileChooser.setMultiSelectionEnabled(true);
		fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		
		addWatchButton = new JButton("Add codesize watch");
		removeWatchButton = new JButton("Remove codesize watch");
		clearButton = new JButton("Clear watches");
		refreshButton = new JButton("Refresh codesizes");
		addWatchButton.addActionListener(this);
		removeWatchButton.addActionListener(this);
		clearButton.addActionListener(this);
		refreshButton.addActionListener(this);
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.add(addWatchButton);
		buttonPanel.add(removeWatchButton);
		buttonPanel.add(clearButton);
		buttonPanel.add(refreshButton);
		buttonPanel.setBorder(BorderFactory.createEtchedBorder());
		add(buttonPanel, BorderLayout.SOUTH);
		table = new JTable(this);
		JScrollPane scroller = new JScrollPane(table);
		JPanel tablePanel = new JPanel(new GridLayout(1, 1));
		tablePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Codesize watches"));
		tablePanel.add(scroller);
		add(tablePanel);
	}
	
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == newWatchItem || e.getSource() == addWatchButton)
		{
			if (fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
			{
				File[] files = fileChooser.getSelectedFiles();
				for (int i=0; i<files.length; i++)
				{
					Codesize.Item item = Codesize.processClassFile(files[i]);
					watches.add(item);
				}
				Collections.sort(watches);
			}
			else
				return;
		}
		else if (e.getSource() == removeWatchButton)
		{
			int[] remove = table.getSelectedRows();
			for (int i=remove.length-1; i>= 0; i--)
				watches.remove(remove[i]);
		}
		else if (e.getSource() == clearButton)
		{
			watches.removeAllElements();
		}
		else if (e.getSource() == refreshButton)
		{
			for (int i=0; i<watches.size(); i++)
			{
				Codesize.Item item = (Codesize.Item)watches.get(i);
				item = Codesize.processClassFile(item.getLocation());
				watches.set(i, item);
			}
			Collections.sort(watches);
		}
		//launch a TableModelEvent now...
		TableModelEvent newEvent = new TableModelEvent(this);
		for (int i=0; i<tmlisteners.size(); i++)
			((TableModelListener)tmlisteners.get(i)).tableChanged(newEvent);
	}
	
	public void addTableModelListener(TableModelListener l)
	{
		tmlisteners.add(l);
	}
	
	public Class getColumnClass(int columnIndex)
	{
		if (columnIndex == 0)
			return java.io.File.class;
		else
			return Integer.class;
	}
	
	public int getColumnCount()
	{
		return 3;
	}
	
	public String getColumnName(int columnIndex)
	{
		if (columnIndex == 0)
			return "Class file";
		else if (columnIndex == 1)
			return "Code size";
		else
			return "Class size";
	}
	
	public int getRowCount()
	{
		return watches.size();
	}
	
	public Object getValueAt(int rowIndex, int columnIndex)
	{
		Codesize.Item row = (Codesize.Item)watches.get(rowIndex);
		if (columnIndex == 0)
			return row.getLocation();
		else if (columnIndex == 1)
			return new Integer(row.getCodeSize());
		else
			return new Integer(row.getClassSize());
	}
	
	public boolean isCellEditable(int rowIndex, int columnIndex)
	{
		return false;
	}
	
	public void removeTableModelListener(TableModelListener l)
	{
		tmlisteners.remove(l);
	}
	
	public void setValueAt(Object aValue, int rowIndex, int columnIndex)
	{
		System.err.println("What are you doing, trying to modify my cells?");
	}
	
	public String getName()
	{
		return "Codesize";
	}
	
	public String getFloodScriptCommand()
	{
		return	  "# This is to help you track codesize without running the command line:\n"
			+ "plugin=kawigi.tools.codesize.CodesizePane(this)\n";
	}
}
