package kawigi.tools.fs;
import java.io.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.text.*;

public class FloodScriptEditor extends JPanel
{
	private DefaultStyledDocument document;
	private FloodScriptLexer colorer;
	private JTextPane textArea;
	private JTabbedPane parent;
	private File f;
	
	public FloodScriptEditor(JTabbedPane parent)
	{
		setLayout(new GridLayout(1, 1));
		this.parent = parent;
		document = new DefaultStyledDocument();
		textArea = new JTextPane(document);
		JList lineNumbers = new JList();
		lineNumbers.setBackground(new Color(128, 128, 192));
		lineNumbers.setForeground(new Color(0, 0, 128));
		Font f = new Font("Monospaced", Font.PLAIN, 12);
		lineNumbers.setFont(f);
		FontMetrics fm = lineNumbers.getFontMetrics(f);
		lineNumbers.setFixedCellHeight(fm.getAscent()+fm.getDescent());
		colorer = new FloodScriptLexer(document, lineNumbers, textArea);
		JPanel fullTextPane = new JPanel(new BorderLayout());
		fullTextPane.add(lineNumbers, BorderLayout.WEST);
		fullTextPane.add(textArea);
		JScrollPane scroll = new JScrollPane(fullTextPane, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		add(scroll);
	}
	
	public void open(File f)
	{
		this.f = f;
		parent.setTitleAt(parent.indexOfComponent(this), f.getName());
		try
		{
			BufferedReader inFile = new BufferedReader(new FileReader(f));
			String str = "";
			String line;
			while ((line = inFile.readLine()) != null)
				str += line + "\n";
			inFile.close();
			textArea.setText(str);
		}
		catch (IOException ex)
		{
		}
	}
	
	public void save(JFileChooser fileChooser, boolean openDialog)
	{
		if (f == null || openDialog)
		{
			if (fileChooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
			{
				f = fileChooser.getSelectedFile();
				parent.setTitleAt(parent.indexOfComponent(this), f.getName());
			}
			else
				JOptionPane.showMessageDialog(this, "Must choose a filename to save.", "Error", JOptionPane.INFORMATION_MESSAGE);
		}
		try
		{
			PrintWriter outFile = new PrintWriter(new FileWriter(fileChooser.getSelectedFile()));
			outFile.print(textArea.getText());
			outFile.close();
		}
		catch (IOException ex)
		{
			JOptionPane.showMessageDialog(this, "I/O Exception: " + ex.getMessage(), "Error - Couldn't write to file", JOptionPane.ERROR_MESSAGE);
		}
		colorer.notifySave();
	}
	
	public boolean close()
	{
		if (colorer.needsSave())
		{
			int result = JOptionPane.showConfirmDialog(this, "Save changes " + (f == null ? "" : "for " + f.getName()) + "?", "Save changes?", JOptionPane.YES_NO_CANCEL_OPTION);
			if (result == JOptionPane.CANCEL_OPTION)
				return false;
			else if (result == JOptionPane.YES_OPTION)
				save(new JFileChooser(), false);
			return true;
		}
		else
			return true;
	}
}