package kawigi.tools.fs;
import java.util.*;
import java.io.*;
import java.lang.reflect.*;

public class FloodScriptParser
{
	private int index;
	private Object thisRef;
	private HashMap results;
	
	public FloodScriptParser(Object thisRef)
	{
		this.thisRef = thisRef;
		results = new HashMap();
	}

	public void loadSettings(File file)
	{
		int linenum = 0;
		String line = null;
		try
		{
			BufferedReader inFile = new BufferedReader(new FileReader(file));
			Vector segs = new Vector();
			while ((line = inFile.readLine()) != null)
			{
				linenum++;
				line = line.trim();
				if (line.indexOf('#') != 0 && line.length() > 0)
				{
					if (line.indexOf('=') >= 0)
					{
						String parameterName = line.substring(0, line.indexOf('=')).trim().toLowerCase();
						index = line.indexOf('=')+1;
						Object o = createObject(line);
						if (results.containsKey(parameterName))
						{
							Object currentObject = results.get(parameterName);
							if (currentObject instanceof Vector)
								((Vector)currentObject).add(o);
							else
							{
								Vector v = new Vector();
								v.add(currentObject);
								v.add(o);
								results.put(parameterName, v);
							}
						}
						else
							results.put(parameterName, o);
					}
					else if (line.toLowerCase().startsWith("load"))
						loadSettings(new File(file, line.substring(line.indexOf(' ')).trim()));
					else if (line.toLowerCase().startsWith("print"))
					{
						index = line.indexOf(' ');
						Object o = createObject(line);
						if (o instanceof Vector)
						{
							Vector v = (Vector)o;
							System.out.print("{" + v.elementAt(0));
							for (int i=1; i<v.size(); i++)
								System.out.print("," + v.elementAt(i));
							System.out.println("}");
						}
						else
							System.out.println(o);
					}
				}
			}
			inFile.close();
		}
		catch(Exception e)
		{
			System.out.println(e);
			System.out.println("(" + file + ", line " + linenum + ": " + line + ")");
			e.printStackTrace();
		}
	}
	
	public Object getObject(String name)
	{
		Object ret = results.get(name.toLowerCase());
		if (ret != null && ret instanceof Vector)
			return ((Vector)ret).elementAt(((Vector)ret).size()-1);
		return ret;
	}
	
	public Object getObject(String name, Object defaultValue)
	{
		if (results.containsKey(name.toLowerCase()))
			return getObject(name.toLowerCase());
		else
			return defaultValue;
	}
	
	public Vector getObjectList(String name)
	{
		if (!results.containsKey(name.toLowerCase()))
			return new Vector();
		else if (results.get(name.toLowerCase()) instanceof Vector)
			return (Vector)results.get(name.toLowerCase());
		else
		{
			Vector ret = new Vector();
			ret.add(results.get(name.toLowerCase()));
			return ret;
		}
		
	}
	
	public Object createObjectFromCommand(String command) throws Exception
	{
		index = 0;
		return createObject(command);
	}
	
	private Object createObject(String command) throws Exception
	{
		while (Character.isWhitespace(command.charAt(index)))
			index++;
		if (command.charAt(index) == '[')
		{
			index++;
			switch (command.charAt(index))
			{
				case 'i':
					int start = index+1;
					index = command.indexOf(']', start);
					if (index == -1)
						throw new Exception("Unclosed int");
					Integer intobj = new Integer(command.substring(start, index));
					index++;
					return intobj;
				case 's':
					start = index+1;
					index = command.indexOf(']', start);
					if (index == -1)
						throw new Exception("Unclosed short");
					Short shortobj = new Short(command.substring(start, index));
					index++;
					return shortobj;
				case 'b':
					start = index+1;
					index = command.indexOf(']', start);
					if (index == -1)
						throw new Exception("Unclosed byte");
					Byte byteobj = new Byte(command.substring(start, index));
					index++;
					return byteobj;
				case 'l':
					start = index+1;
					index = command.indexOf(']', start);
					if (index == -1)
						throw new Exception("Unclosed long");
					Long longobj = new Long(command.substring(start, index));
					index++;
					return longobj;
				case 'f':
					start = index+1;
					index = command.indexOf(']', start);
					if (index == -1)
						throw new Exception("Unclosed float");
					Float floatobj = new Float(command.substring(start, index));
					index++;
					return floatobj;
				case 'd':
					start = index+1;
					index = command.indexOf(']', start);
					if (index == -1)
						throw new Exception("Unclosed double");
					Double doubleobj = new Double(command.substring(start, index));
					index++;
					return doubleobj;
				case '#':
					Boolean bool;
					if (command.charAt(index+1) == 'f')
						bool = new Boolean(false);
					else if (command.charAt(index+1) == 't')
						bool = new Boolean(true);
					else
						throw new Exception ("Invalid boolean");
					index = command.indexOf(']', index) + 1;
					return bool;
				case '\'':
					index++;
					Character chobj;
					if (command.charAt(index) == '\\')
					{
						chobj = new Character(interpretEscapeSequence(command));
					}
					else
						chobj = new Character(command.charAt(index));
					index = command.indexOf(']', index) + 1;
					return chobj;
				case '\"':
					String str = "";
					index++;
					while (command.charAt(index) != '\"')
						if (command.charAt(index) == '\\')
						{
							str += interpretEscapeSequence(command);
						}
						else
						{
							str += command.charAt(index);
							index++;
						}
					index = command.indexOf(']', index)+1;
					return str;
				default:
					start = index+1;
					index = command.indexOf(']', start);
					if (index == -1)
						throw new Exception("Unclosed literal");
					str = command.substring(start, index);
					index++;
					try
					{
						intobj = new Integer(str);
						return intobj;
					}
					catch (NumberFormatException ex2)
					{
						try
						{
							doubleobj = new Double(str);
							return doubleobj;
						}
						catch (NumberFormatException ex)
						{
							throw new Exception("Unrecognized literal type");
						}
					}
			}
		}
		else
		{
			String className = "";
			while (index < command.length() && (Character.isLetterOrDigit(command.charAt(index)) || command.charAt(index) == '_' || command.charAt(index) == '.' || command.charAt(index) == '$'))
				className += command.charAt(index++);
			while (index < command.length() && Character.isWhitespace(command.charAt(index)))
				index++;
			if (className.equals("this"))
				return thisRef;
			else if (className.equals("null"))
				return null;
			else if (results.containsKey(className))
				return results.get(className);
			if (index != command.length() && command.charAt(index) == '(')
			{
				Vector args = new Vector();
				do
				{
					index++;
					args.add(createObject(command));
				}
				while (command.charAt(index) == ',');
				Object[] argarray = args.toArray();
				Constructor[] constructors = Class.forName(className).getConstructors();
				for (int i=0; i<constructors.length; i++)
				{
					try
					{
						return constructors[i].newInstance(argarray);
					}
					catch (InvocationTargetException ex){ex.getCause().printStackTrace();}
					catch (Exception ex){System.out.println(ex);}
				}
				for (int i=0; i<args.size(); i++)
					System.out.println(args.elementAt(i));
				throw new Exception("Constructor not found for given parameters of " + className);
			}
			else
				return Class.forName(className).newInstance();
		}
			
	}
	
	private char interpretEscapeSequence(String command)
	{
		index++;
		switch (command.charAt(index))
		{
			case 'n':
				index++;
				return '\n';
			case 'r':
				index++;
				return '\r';
			case 't':
				index++;
				return '\t';
			case 'b':	//think anyone will use this?
				index++;
				return '\b';
			case '\\':
				index++;
				return '\\';
			case '\'':
				index++;
				return '\'';
			case '\"':
				index++;
				return '\"';
			default:
				if (Character.isDigit(command.charAt(index)))
				{
					char ch = (char)Integer.parseInt(command.substring(index, command.indexOf('\'', index)));
					index = command.indexOf('\'', index)+1;
					return ch;
				}
				else
					return command.charAt(index++);
		}
	}
	
	public static void main(String args[]) throws Exception
	{
		FloodScriptParser parser = new FloodScriptParser(null);
		parser.loadSettings(new File(args[0]));
	}
}
