package kawigi.tools.graph;
import kawigi.tools.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class GraphControlWindow extends JInternalFrame implements ItemListener, ActionListener, InternalFrameListener
{
	private static int created;
	private SegmentedData data;
	private GraphDisplayer disp;
	private GraphColorSelector colorselector;
	private JButton refreshButton, deleteButton;
	private FileFormat sourceFormat;
	private File sourceFile;
	
	public GraphControlWindow(SegmentedData data, String name, GraphDisplayer disp, FileFormat sourceFormat, File sourceFile)
	{
		super(name, true, true, true, true);
		this.sourceFormat = sourceFormat;
		this.sourceFile = sourceFile;
		this.data = data;
		this.disp = disp;
		setLocation(created*10, created*10);
		setFrameIcon(new ImageIcon(Toolkit.getDefaultToolkit().getImage("floodgrapher.gif")));
		created++;
		colorselector = new GraphColorSelector();
		colorselector.addActionListener(this);
		getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
		getContentPane().add(colorselector);
		JPanel buttonPanel = new JPanel(new GridLayout(1, 2));
		refreshButton = new JButton("Refresh");
		deleteButton = new JButton("Delete File");
		buttonPanel.add(refreshButton);
		buttonPanel.add(deleteButton);
		refreshButton.addActionListener(this);
		deleteButton.addActionListener(this);
		getContentPane().add(buttonPanel);
		data.createSelectionGUI(this);
		addInternalFrameListener(this);
		setVisible(true);
		pack();
		show();
	}
	
	public void actionPerformed(ActionEvent event)
	{
		if (event.getSource() == refreshButton)
		{
			try
			{
				data.setData(sourceFormat.getData(sourceFile));
			}
			catch (Exception e)
			{
			}
			GraphData gdata = data.getSelectedGraphData();
			gdata.setColor(colorselector.getSelectedColor());
			disp.addGraph(gdata, this);
		}
		else if (event.getSource() == deleteButton)
		{
			if (!sourceFile.delete())
				JOptionPane.showInternalMessageDialog(this, "Could not delete file.", "Error", JOptionPane.ERROR_MESSAGE);
		}
		else
		{
			GraphData gdata = data.getSelectedGraphData();
			gdata.setColor(colorselector.getSelectedColor());
			disp.addGraph(gdata, this);
		}
	}
	
	public void itemStateChanged(ItemEvent event)
	{
		GraphData gdata = data.getSelectedGraphData();
		gdata.setColor(colorselector.getSelectedColor());
		disp.addGraph(gdata, this);
	}
	
	public void internalFrameClosing(InternalFrameEvent e)
	{
		disp.removeGraph(this);
		dispose();
	}

	public void internalFrameActivated(InternalFrameEvent e){}
	public void internalFrameClosed(InternalFrameEvent e){}
	public void internalFrameDeactivated(InternalFrameEvent e){}
	public void internalFrameDeiconified(InternalFrameEvent e){}
	public void internalFrameIconified(InternalFrameEvent e){}
	public void internalFrameOpened(InternalFrameEvent e){}
}
