package kawigi.tools.graph;
import kawigi.tools.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;

public class NewGraphWindow extends JDialog implements ActionListener
{
	private JDesktopPane callback;
	private GraphDisplayer displayer;
	private File baseDirectory;
	private JButton okButton, cancelButton, browseButton;
	private static JFileChooser fileChooser;
	private JComboBox formatBox;
	private JTextField fileField;
	private FileFormat[] formats;
	
	public NewGraphWindow(JDesktopPane desktop, GraphDisplayer displayer, Frame owner, FileFormat[] formats, File defaultDirectory)
	{
		super(owner, "Open Graph Data", true);
		callback = desktop;
		this.formats = formats;
		this.displayer = displayer;
		baseDirectory = defaultDirectory;
		
		okButton = new JButton("OK");
		cancelButton = new JButton("Cancel");
		browseButton = new JButton("Browse...");
		formatBox = new JComboBox();
		for (int i=0; i<formats.length; i++)
			formatBox.addItem(formats[i].getFormatName());
		fileField = new JTextField(20);
		if (fileChooser == null)
			fileChooser = new JFileChooser(defaultDirectory);
		
		okButton.addActionListener(this);
		cancelButton.addActionListener(this);
		browseButton.addActionListener(this);
		fileField.addActionListener(this);
		
		JPanel filePanel = new JPanel();
		filePanel.setLayout(new BoxLayout(filePanel, BoxLayout.X_AXIS));
		filePanel.add(new JLabel("File:"));
		filePanel.add(fileField);
		filePanel.add(browseButton);
		
		JPanel formatPanel = new JPanel();
		formatPanel.setLayout(new BoxLayout(formatPanel, BoxLayout.X_AXIS));
		formatPanel.add(new JLabel("Format:"));
		formatPanel.add(formatBox);
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.add(okButton);
		buttonPanel.add(cancelButton);
		
		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
		mainPanel.add(filePanel);
		mainPanel.add(formatPanel);
		mainPanel.add(buttonPanel);
		
		getContentPane().add(mainPanel);
		pack();
		show();
	}
	
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == okButton || e.getSource() == fileField)
		{
			if (formatBox.getSelectedIndex() < formats.length)
			{
				try
				{
					SegmentedData data = formats[formatBox.getSelectedIndex()].getData(fileChooser.getSelectedFile());
					String filename = fileField.getText();
					File file = new File(filename);
					if (filename.indexOf(File.separatorChar) >= 0)
						filename = filename.substring(filename.lastIndexOf(File.separatorChar)+1);
					GraphControlWindow window = new GraphControlWindow(data, filename + " - " + formatBox.getSelectedItem(), displayer, formats[formatBox.getSelectedIndex()], file);
					callback.add(window);
				}
				catch (Exception ex)
				{
					System.err.println(ex);
					ex.printStackTrace();
					return;
				}
			}
			dispose();
		}
		else if (e.getSource() == cancelButton)
			dispose();
		else if (e.getSource() == browseButton)
		{
			if (fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
				fileField.setText(fileChooser.getSelectedFile().getPath());
		}
	}
}
