package kawigi.tools.help;
import javax.swing.*;
import kawigi.tools.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Vector;
import javax.swing.event.*;
import javax.swing.text.html.*;

public class HelpPanel extends FloodGrapherGUIPlugin implements ActionListener, HyperlinkListener
{	
	private HTMLViewer display;
	private File helpdirectory, currentPage;
	private Vector history, future;
	private JButton backButton, forwardButton, contentsButton, refreshButton;
	private JLabel statusLabel;
	private JScrollPane spane;
	private String defaultLabel;
	
	public HelpPanel(String helpdirectory)
	{
		setSize(500, 600);
		history = new Vector();
		future = new Vector();
		this.helpdirectory = new File(helpdirectory);
		currentPage = new File(this.helpdirectory, "index.html");
		statusLabel = new JLabel("Done.");
		backButton = new JButton(new ImageIcon("kawigi" + File.separator + "resource" + File.separator + "navigation" + File.separator + "Back24.gif"));
		forwardButton = new JButton(new ImageIcon("kawigi" + File.separator + "resource" + File.separator + "navigation" + File.separator + "Forward24.gif"));
		contentsButton = new JButton(new ImageIcon("kawigi" + File.separator + "resource" + File.separator + "navigation" + File.separator + "Home24.gif"));
		JPanel buttonPanel = new JPanel(new GridLayout(1, 4));
		buttonPanel.add(backButton);
		buttonPanel.add(forwardButton);
		buttonPanel.add(contentsButton);
		JPanel shrinkPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));	//this is to make my buttons stay the same size, left aligned, regardless of the window size
		shrinkPanel.add(buttonPanel);
		backButton.addActionListener(this);
		forwardButton.addActionListener(this);
		contentsButton.addActionListener(this);
		display = new HTMLViewer();
		display.addHyperlinkListener(this);
		spane = new JScrollPane(display, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		spane.setWheelScrollingEnabled(true);
		setLayout(new BorderLayout());
		add(shrinkPanel, BorderLayout.NORTH);
		add(statusLabel, BorderLayout.SOUTH);
		add(spane);
		setPageFromContext(null);
	}
	
	public void setPageByName(String filename, boolean relativelink)
	{
		statusLabel.setText("Loading page...");
		if (currentPage != null)
			history.add(currentPage);
		currentPage = new File(relativelink ? currentPage.getParentFile() : helpdirectory, filename);
		String result = display.goToPage(currentPage);
		if (result.equals(""))
		{
			future.removeAllElements();
			defaultLabel = "Done.";
		}
		else
		{
			currentPage = (File)history.remove(history.size()-1);
			defaultLabel = result;
		}
		statusLabel.setText(defaultLabel);
	}
	
	public void setPageFromContext(String context)
	{
		statusLabel.setText("Finding Page...");
		setPageByName(pageFromContext(context), false);
	}
	
	private static String pageFromContext(String context)
	{
		if (context == null || context.length() == 0)
			return "index.html";
		if (context.endsWith(".html"))
			return context;
		return "index.html";
	}
	
	public void refresh()
	{
		statusLabel.setText("Loading page...");
		String result = display.goToPage(currentPage);
		if (result.length() != 0)
			defaultLabel = result;
		else
			defaultLabel = "Done.";
			
		statusLabel.setText(defaultLabel);
	}
	
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == backButton)
		{
			if (history.size() > 0)
			{
				future.add(currentPage);
				currentPage = (File)history.remove(history.size()-1);
			}
			refresh();
		}
		else if (e.getSource() == forwardButton)
		{
			if (future.size() > 0)
			{
				history.add(currentPage);
				currentPage = (File)future.remove(future.size()-1);
			}
			refresh();
		}
		else if (e.getSource() == contentsButton)
			setPageFromContext(null);
	}
	
	public void hyperlinkUpdate(HyperlinkEvent e)
	{
		if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
			setPageByName(e.getDescription(), true);
		else if (e.getEventType() == HyperlinkEvent.EventType.ENTERED)
			statusLabel.setText(e.getDescription());
		else if (e.getEventType() == HyperlinkEvent.EventType.EXITED)
			statusLabel.setText(defaultLabel);
	}
	
	public String getName()
	{
		return "Help";
	}
	
	public String getFloodScriptCommand()
	{
		String directory = helpdirectory.toString();
		//way too many levels of escape sequences...
		directory = directory.replaceAll("\\\\", "\\\\\\\\");
		return	  "# The help plugin:\n"
			+ "plugin=kawigi.tools.help.HelpPanel([\"" + directory + "\"])\n";
	}
}
