package kawigi.tools.parts;
import robocode.*;

/**
 * Some movement systems may allow the use of a Distancing Algorithm, which dictates whether it is
 * better to move toward or away from the enemy or stay at the same distance.
 **/
public interface DistancingAlgorithm
{
	/**
	 * Should return a negative number if the robot should move closer, a positive number if 
	 * the robot should move away, or zero if it should stay at approximately the same distance.
	 **/
	public int getDistanceDirection();
	
	/**
	 * Called to notify the DistancingAlgorithm of events given to the robot.
	 **/
	public void onEvent(Event e);
}
