package kawigi.tools.parts;
import robocode.*;

public class DynamicDistancing implements DistancingAlgorithm
{
	private double[] mygain, myloss, enemygain, enemyloss;
	private double theirEnergy, distance, segmentSize;
	private int distanceindex;

	public DynamicDistancing(double segmentSize)
	{
		this.segmentSize = segmentSize;
		int arraylength = (int)Math.ceil(1200/segmentSize);
		mygain = new double[arraylength];
		myloss = new double[arraylength];
		enemygain = new double[arraylength];
		enemyloss = new double[arraylength];
		theirEnergy = 100;
	}

	public DynamicDistancing()
	{
		this(50);
	}
	
	public int getDistanceDirection()
	{
		int bestindex = 1;
		for (int i=1; i<mygain.length-1; i++)
			if (findBenefit2(i) > findBenefit2(bestindex))
				bestindex = i;
		double minDistance = bestindex*segmentSize-segmentSize/2;
		double maxDistance = (bestindex+1)*segmentSize+segmentSize/2;
		if (distance < minDistance)
			return 1;
		else if (distance > maxDistance)
			return -1;
		else
			return 0;
	}

	public double findBenefit2(int index)
	{
		return findBenefit(index-1)+2*findBenefit(index)+findBenefit(index+1);
	}

	public double findBenefit(int index)
	{
		return (mygain[index]+enemyloss[index]-myloss[index]-enemygain[index]);
	}
	
	public void onEvent(Event e)
	{
		if (e instanceof BulletHitBulletEvent)
		{
			double power = ((BulletHitBulletEvent)e).getBullet().getPower();
			myloss[distanceindex] += power;
		}
		else if (e instanceof BulletHitEvent)
		{
			double power = ((BulletHitEvent)e).getBullet().getPower();
			myloss[distanceindex] += power;
			mygain[distanceindex] += power*3;
			enemyloss[distanceindex] += Math.max(4*power, 6*power-2);
			theirEnergy -= Math.max(4*power, 6*power-2);
		}
		else if (e instanceof BulletMissedEvent)
		{
			double power = ((BulletMissedEvent)e).getBullet().getPower();
			myloss[distanceindex] += power;
		}
		else if (e instanceof HitByBulletEvent)
		{
			double power = ((HitByBulletEvent)e).getPower();
			enemygain[distanceindex] += power*3;
			theirEnergy += power*3;
			myloss[distanceindex] += Math.max(4*power, 6*power-2);
		}
		else if (e instanceof ScannedRobotEvent)
		{
			ScannedRobotEvent scan = (ScannedRobotEvent)e;
			distance = scan.getDistance();
			distanceindex = (int)Math.min(mygain.length-1, distance/segmentSize);
			double energydiff = theirEnergy-scan.getEnergy();
			theirEnergy = scan.getEnergy();
			if (energydiff > 0.09 && energydiff <= 3)
				enemyloss[distanceindex] += energydiff;
		}
		else if (e instanceof WinEvent || e instanceof DeathEvent)
		{
			theirEnergy = 100;
		}
	}
}

