package kawigi.tools.parts;
import robocode.*;
import java.awt.geom.*;

public class FloodMovement implements Movement
{
	private DistancingAlgorithm dist;
	private double speed, currentDirection, currentEnergy, ex, ey;
	private long nextTime;
	private Rectangle2D fieldRect;
	
	public FloodMovement(DistancingAlgorithm dist)
	{
		this.dist = dist;
		currentDirection = 1;
		speed = 0;
		currentEnergy = 100;
	}
	
	public void onEvent(Event e, AdvancedRobot ar)
	{
		if (fieldRect == null)
		{
			fieldRect = new Rectangle2D.Double(18, 18, ar.getBattleFieldWidth()-36, ar.getBattleFieldHeight()-36);
		}
		dist.onEvent(e);
		if (e instanceof WinEvent || e instanceof DeathEvent)
		{
			nextTime = 0;
			currentEnergy = 100;
		}
		else if (e instanceof ScannedRobotEvent)
		{
			ScannedRobotEvent scan = (ScannedRobotEvent)e;
			double energydiff = currentEnergy - scan.getEnergy();
			currentEnergy = scan.getEnergy();
			if (energydiff > 0.09 && energydiff <= 3)
			{
				if (ar.getTime() > nextTime)
				{
					nextTime = (long)(ar.getTime() + scan.getDistance()/(20-energydiff*3)/2);
					speed = 2+Math.random()*20;
					currentDirection = Math.random() < .5 ? -1 : 1;
				}
			}
			ex = ar.getX() + Math.sin(ar.getHeadingRadians()+scan.getBearingRadians())*scan.getDistance();
			ey = ar.getY() + Math.cos(ar.getHeadingRadians()+scan.getBearingRadians())*scan.getDistance();
		}
		else if (e instanceof BulletHitEvent)
		{
			double power = ((BulletHitEvent)e).getBullet().getPower();
			currentEnergy -= Math.max(4*power, 6*power-2);
		}
		else if (e instanceof HitByBulletEvent)
		{
			currentEnergy += ((HitByBulletEvent)e).getPower()*3;
		}
	}

	public void run(AdvancedRobot ar)
	{
		if (fieldRect == null)
			fieldRect = new Rectangle2D.Double(18, 18, ar.getBattleFieldWidth()-36, ar.getBattleFieldHeight()-36);
		double absBearing = Math.atan2(ex-ar.getX(), ey-ar.getY());
		double moveDirection = absBearing-(currentDirection*Math.PI/2);
		int distadjust = dist.getDistanceDirection();
		
		if (!fieldRect.contains(ar.getX()+Math.sin(Math.PI+absBearing)*50, ar.getY()+Math.cos(Math.PI+absBearing)*50) || distadjust < 0 || ar.getEnergy()/currentEnergy >= 7 || Math.min(ar.getX(), ar.getBattleFieldWidth()-ar.getX()) + Math.min(ar.getY(), ar.getBattleFieldHeight()-ar.getY()) < 250)
			moveDirection += currentDirection*Math.PI/6;
		else if (distadjust > 0 || currentEnergy/ar.getEnergy() >= 7)
			moveDirection -= currentDirection*Math.PI/6;
		if (!fieldRect.contains(ar.getX()+Math.sin(moveDirection)*30, ar.getY()+Math.cos(moveDirection)*30))
			currentDirection = -currentDirection;
		ar.setMaxVelocity(speed);
		double turn = robocode.util.Utils.normalRelativeAngle(moveDirection-ar.getHeadingRadians());
		double direction = 1;
		if (Math.abs(turn) > Math.PI/2)
		{
			direction = -1;
			if (turn < 0)
				turn += Math.PI;
			else
				turn -= Math.PI;
		}
		ar.setTurnRightRadians(turn);
		ar.setAhead(direction*100);
	}
}
